
package com.aukcjomat.parser;

import org.htmlparser.*;
import org.htmlparser.util.*;

/**
 * Moj interfejs do drzewek HTMLa. Kazdemu wezlowi (takze tekstowi i komentarzowi)
 * odpowiadaja dwa obiekty Cos, jeden to poczatek, drugi to koniec.
 * Wszystkie obiekty z calego pliku sa polaczone w liste.
 *
 * @author Pawel Parys
 */

public class Cos
{
    static Cos dummy = new Cos(null, null, true);
    /**
     * Nastepny Cos na liscie.
     */
    public Cos next = null;
    /**
     * Poprzedni Cos na liscie.
     */
    public Cos prev = null;
    /**
     * Dla Cosia zamykajacego jest to odpowiadajacy mu Cos zamykajacy i odwrotnie.
     */
    public Cos pair;
    /**
     * Ojciec.
     */
    public Cos up;
    /**
     * Odpowiadajacy obiekt Node zwrocony przez parser htmla.
     */
    public Node node;
    /**
     * Czy jest to obiekt otwierajacy.
     */
    public boolean first;
    public Cos(Cos up, Node node, boolean first) { this.up = up; this.node = node; this.first = first; }
    /**
     * Numer obiektu. Obiekty na liscie sa ponumerowane w kolejnosci rosnacej.
     */
    public int nr = 1000000000;

    Cos get_last()
    {
        Cos cos = this;
        while (cos.next!=null)
            cos = cos.next;
        return cos;
    }


    static boolean czy_litera(char ch)
    {
        return Character.isLetterOrDigit(ch) || ch==',';
    }
    
    /**
     * Sprawdza czy tekst wewnatrz tego taga nie jest trescia strony.
     * @return Prawda dla tagow otwierajacych typu SCRIPT, STYLE, SELECT.
     */
    public boolean czy_pominac()
    {
        return Tools.isTag(node) && (Tools.tagIs((Tag)node, "SCRIPT") || 
                Tools.tagIs((Tag)node, "STYLE") || 
                Tools.tagIs((Tag)node, "SELECT")) && first;
    }
    
    /**
     * Wyszukuje tekstu na liscie Cosi w podanym zakresie. Pomija wnetrze elementow SCRIPT, STYLE i SELECT.
     * @param s Szukany tekst.
     * @param start Pierwszy uwzgledniany obiekt.
     * @param end Ostatni uwzgleniany obiekt.
     * @return Pierszy Cos, ktory zawiera podany tekst.
     */
    public static Cos znajdz_str(String s, Cos start, Cos end)
    {
        if (s.equals(""))
            return null;
        for (Cos c = start; c.nr<=end.nr; c = c.next)
        {
            if (c.czy_pominac())
            {
                c = c.pair;
                continue;
            }
            if (Tools.isText(c.node) && c.first)
            {
                String s2 = c.node.getText();
                int x = s2.indexOf(s);
                if (x<0) continue;
                if (x>0 && czy_litera(s2.charAt(x-1))) continue;
                int x2 = x+s.length();
                if (x2<s2.length() && czy_litera(s2.charAt(x2))) continue;
                return c;
            }
        }
        return null;
    }
    
    /**
     * Wyszukuje obrazka o zadanym alt na liscie Cosi w podanym zakresie. 
     * @param s Szukany tekst.
     * @param start Pierwszy uwzgledniany obiekt.
     * @param end Ostatni uwzgleniany obiekt.
     * @return Pierszy Cos, ktory zawiera podany tekst w alt obrazka.
     */
    public static Cos znajdz_img(String s, Cos start, Cos end)
    {
        //System.out.println("Szukam:"+s);
        if (s.equals(""))
            return null;
        for (Cos c = start; c.nr<=end.nr; c = c.next)
            if (Tools.isTagPossiblyEmpty(c.node) && c.first && Tools.tagIs((Tag)c.node, "IMG") &&
                    Tools.klasa((Tag)c.node, s))
                return c;
        return null;
    }

    /**
     * Wyszukuje komentarz o zadanej tresci.
     * @param s Szukana tresc komentarza.
     * @param start Pierwszy uwzgledniany obiekt.
     * @param end Ostatni uwzgleniany obiekt.
     * @return Cos pierwszego komentarza, ktorego tresc jest rowna podanemu tekstowi.
     */
    public static Cos znajdz_remark(String s, Cos start, Cos end)
    {
        if (s.equals(""))
            return null;
        for (Cos c = start; c.nr<=end.nr; c = c.next)
            if (Tools.isRemark(c.node) && c.first && c.node.getText().equals(s))
                return c;
        return null;
    }
    
    /**
     * Wyszukuje tekstu na liscie Cosi wewnatrz siebie (wraz z soba). Pomija wnetrze elementow SCRIPT i SELECT.
     * @param s Szukany tekst.
     * @return Pierszy Cos, ktory zawiera podany tekst.
     */
    public Cos znajdz_str(String s)
    {
        return znajdz_str(s, this, this.pair);
    }
    /**
     * Wyszukuje komentarz o zadanej tresci na liscie Cosi wewnatrz siebie (wraz z soba).
     * @param s Szukana tresc komentarza.
     * @return Cos pierwszego komentarza, ktorego tresc jest rowna podanemu tekstowi.
     */
    public Cos znajdz_remark(String s)
    {
        return znajdz_remark(s, this, this.pair);
    }


    static Cos przyg_dla_it(NodeIterator it, Cos up) throws ParserException
    {
        Cos cos = null, cos2 = null;
        while (it.hasMoreNodes())
        {
            Node n = it.nextNode();
            Cos front = new Cos(up, n, true);
            Cos back = new Cos(up, n, false);
            front.pair = back;
            back.pair = front;
            if (n.getChildren()!=null)
            {
                Cos srodek = przyg_dla_it(n.getChildren().elements(), front);
                front.next = srodek;
                srodek.prev = front;
                srodek = srodek.get_last();
                srodek.next = back;
                back.prev = srodek;
            }
            else
            {
                front.next = back;
                back.prev = front;
            }
            front.prev = cos;
            if (cos==null)
                cos2 = front;
            else
                cos.next = front;
            cos = back;
        }
        return cos2;
    }

    /**
     * Tworzy strukture Cosi na podstawie elementow zwroconych przez parser
     * htmla.  Kazdemu wezlowi (takze tekstowi i komentarzowi) odpowiadaja dwa obiekty
     * Cos, jeden to poczatek, drugi to koniec.  Wszystkie obiekty z calego
     * pliku sa polaczone w liste.  Dodawany jest specjalny element glowny,
     * ktory zawiera w sobie wszystkie elementy, ma ustawione node na null. 
     * Ponadto za nim dodawany jest wezel zaslepka, dzieki czemu mozna
     * zawsze porownywac z .next.nr.
     * @param nodes Glowny iterator po wezlach z parsera htmla.
     * @return Glowny obiekt Cos.
     */
    public static Cos przygotuj(NodeIterator nodes) throws ParserException
    {
        if (nodes==null) return null;
        Cos main_cos = new Cos(null, null, true);
        Cos end_cos = new Cos(null, null, false);
        Cos c = przyg_dla_it(nodes, main_cos);
        main_cos.pair = end_cos;
        end_cos.pair = main_cos;
        main_cos.next = c;
        c.prev = main_cos;
        c = c.get_last();
        c.next = end_cos;
        end_cos.prev = c;
        int nr = 0;
        end_cos.next = dummy;
        for (c = main_cos; c!=dummy; c = c.next)
          c.nr = nr++;
        return main_cos;
    }

};
