package com.aukcjomat.parser;

import org.htmlparser.*;
import org.htmlparser.util.*;
import org.htmlparser.lexer.*;
import java.util.*;
import java.lang.*;
import java.text.*;
import com.aukcjomat.komentarz.*;
import com.aukcjomat.*;

public class MadryParser
{

  static int U_max_por = 5;

  NodeIterator sub(Node tag)
  {
    return tag.getChildren().elements();
  }

  boolean tagIs(Tag t, String s)
  {
    return t.getTagName().equalsIgnoreCase(s);
  }
  
  boolean klasa(Tag t, String s)
  {
    String c = t.getAttribute("class");
    return c!=null && c.equals(s);
  }

  static final int C_LISTA_KATEGORII = 0;
  static final int C_LISTA_AUKCJI = 1;
  static final int C_AUKCJA = 2;
  static final int C_USER = 3;
  static final int C_AUKCJA_DLA_KOM = 4;

  long parsuj_czas(String s)
  {
    try
    {
      s = s.trim();
      int x = s.indexOf(' ')+1;
      int y = s.lastIndexOf(' ');
      if (y<0) y = s.length();
      s = s.substring(x, y);
      Locale l = new Locale("pl");
      DateFormat df = new SimpleDateFormat("dd MMM yyyy HH:mm:ss", l);     
      return df.parse(s).getTime();
    }
    catch (ParseException e)
    {
      System.out.println("Blad: " + e.toString());
      e.printStackTrace();
      return 0;
    }
  }

  String popraw_czas(String s)
  {
    return ""+parsuj_czas(s);
  }

  String popraw_cene(String s)
  {
    String ret = "";
    for (int a = 0; a<s.length(); ++a)
    {
      char ch = s.charAt(a);
      if (ch==' ') continue;
      if (ch==',') ch = '.';
      ret += ch;
    }
    return ret.trim();
  }

  String popraw_kreske(String s)
  {
    return s.equals("-") ? "0" : s;
  }

  BazaKomentarzy baza = new BazaKomentarzy();

  void wyswietl()
  {
    Iterator it = ret_of.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry e = (Map.Entry)it.next();
      System.out.println(e.getKey()+" = "+e.getValue());
    }
  }


  abstract class Warunek 
  {
    abstract public boolean czy(Tag t) throws ParserException;
    public void ok(Tag t) throws ParserException {};
    public void after(Tag t) throws ParserException {};
    public void remark(Remark t) throws ParserException {};
    public void text(Text t) throws ParserException {};
  };

  boolean isTagPossiblyEmpty(Node node)
  {
    return node instanceof Tag;
  }

  boolean isTag(Node node)
  {
    return isTagPossiblyEmpty(node) && node.getChildren()!=null;
  }

  boolean isText(Node node)
  {
    return node instanceof Text;
  }

  boolean isRemark(Node node)
  {
    return node instanceof Remark;
  }

  void znajdz_anal(Node node, Warunek war) throws ParserException
  {
      if (isRemark(node))
        war.remark((Remark)node);
      if (isText(node))
        war.text((Text)node);
      if (!isTag(node))
        return;
      Tag tag = (Tag)node;
      if (war.czy(tag))
        war.ok(tag);
      else
        znajdz(sub(node), war);
      war.after(tag);
  }

  void znajdz(NodeIterator it, Warunek war) throws ParserException
  {
    while (it.hasMoreNodes())
    {
      znajdz_anal(it.nextNode(), war);
    }
  }

  boolean znajdz_anal_wew(Node node, Warunek war) throws ParserException
  {
      if (!isTag(node))
        return false;
      if (znajdz_wewnetrzne(sub(node), war))
        return true;
      Tag tag = (Tag)node;
      if (war.czy(tag))
      {
        war.ok(tag);
        return true;
      }
      return false;
  }

  boolean znajdz_wewnetrzne(NodeIterator it, Warunek war) throws ParserException
  {
    boolean ret = false;
    while (it.hasMoreNodes())
    {
      ret = znajdz_anal_wew(it.nextNode(), war) || ret;
    }
    return ret;
  }

  String ret_str;

  String rozpoznaj_linka(NodeIterator it) throws ParserException
  {
    ret_str = "";
    znajdz(it, new Warunek()
    {
      public boolean czy(Tag tag) throws ParserException
      {
        if (!ret_str.equals("") || !tagIs(tag, "A"))
          return false;
        ret_str = tag.getAttribute("href");
        return true;
      }
    });
    return absoluteUrl(ret_str);
  }


  String rozpoznaj_tresc_anal(Node node) throws ParserException
  {
      if (isText(node))
        return ((Text)node).getText();
      if (isTagPossiblyEmpty(node) && tagIs((Tag)node, "BR"))
        return " ";
      if (!isTag(node))
        return "";
      return rozpoznaj_tresc2(sub(node));
  }

  String rozpoznaj_tresc2(NodeIterator it) throws ParserException
  {
    String ret = "";
    while (it.hasMoreNodes())
    {
      ret = ret + rozpoznaj_tresc_anal(it.nextNode());
    }
    return ret.trim();
  }

  String rozpoznaj_tresc(NodeIterator it) throws ParserException
  {
    return rozpoznaj_tresc2(it).trim();
  }


  String absoluteUrl(String url)
  {
    if (url.equals("") || url.indexOf("http://")>=0)
      return url;
    return "http://www.allegro.pl"+url;
  }


  boolean cyfra(char ch)
  {
    return ch>='0' && ch<='9';
  }

  int do_cyfry(String napis, int x0)
  {
    int x;
    for (x = x0; x<napis.length(); ++x)
      if (cyfra(napis.charAt(x)))
        break;
    return x;
  }

  int do_konca(String napis, int x0)		// znajdz koniec ceny
  {
    int x;
    for (x = x0; x<napis.length(); ++x)
      if (!cyfra(napis.charAt(x)) && napis.charAt(x)!=',' && napis.charAt(x)!=' ')
        break;
    return x;
  }


//////////////////////////////////////////////////
//						//
//		LISTA AUKCJI			//
//						//
//////////////////////////////////////////////////

  Map ret_of;

  void rozpoznaj_ceny_na_liscie(String napis)
  {
    int y = napis.indexOf("teraz");
    int s1 = do_cyfry(napis, 0);
    int k1 = do_konca(napis, s1);
    int s2 = do_cyfry(napis, k1);
    int k2 = do_konca(napis, s2);
    if (y>=0 && s1>y) // tylko cena kup_teraz
    {
      s2 = s1;
      k2 = k1;
      s1 = k1 = 0;
    }
    ret_of.put("cena", popraw_cene(napis.substring(s1, k1)));
    ret_of.put("cena_kup_teraz", popraw_cene(napis.substring(s2, k2)));
  }

  void analizuj_wiersz_listy_ofert(NodeIterator it) throws ParserException
  {
    ret_of = new HashMap();
    znajdz_wewnetrzne(it, new Warunek()
    {
      int ile = 0;
      public boolean czy(Tag tag) throws ParserException
      {
        return tagIs(tag, "TD");
      }
      public void ok(Tag tag) throws ParserException
      {
        ++ile;
        String napis = rozpoznaj_tresc(sub(tag));
        if (ile==2)
        {
          ret_of.put("url", rozpoznaj_linka(sub(tag)));
          ret_of.put("nazwa", napis);
        }
        if (ile==3)
          rozpoznaj_ceny_na_liscie(napis);
        if (ile==4)
          ret_of.put("ile_ofert", popraw_kreske(napis));
        if (ile==5)
          ret_of.put("do_konca", napis);
      }
    });
    if (ret_of.size()!=6)
      return;
    String url = (String)ret_of.get("url");
    if (!fajny_parser.siets.doLookup(url))
;//!!!      Sciagacz.sciagaj(url, new Rodzaj(C_AUKCJA));
    fajny_parser.siets.doUpdate(url, ret_of);
      wyswietl();
  }


  static final int ile_omin = 2;

  void analizuj_liste_ofert(NodeIterator it) throws ParserException
  {
    int ile = 0;
    while (it.hasMoreNodes())
    {
      Node node = it.nextNode();
      if (!isTag(node)) continue;
      Tag tag = (Tag)node;
      if (!tagIs(tag, "TR")) continue;
      ++ile;
      if (ile>ile_omin)
        analizuj_wiersz_listy_ofert(sub(tag));
    }
  }

  String nast;

  void znajdz_nastepne(NodeIterator it) throws ParserException
  {
    znajdz(it, new Warunek()
    {
        public boolean czy(Tag tag) throws ParserException
        {
          if (tagIs(tag, "A") && 
            rozpoznaj_tresc(sub(tag)).indexOf("Nast")>=0)
          {
            nast = tag.getAttribute("href");
            return true;
          }
          return false;
        }
    });
  }

    void parsuj_liste_ofert(NodeIterator it) throws ParserException
    {
      nast = "";
      znajdz(it, new Warunek()
      {
        public boolean czy(Tag tag) throws ParserException
        {
          if (tagIs(tag, "TABLE") && (klasa(tag, "listing_f") || klasa(tag, "listing_r")))
          {
            analizuj_liste_ofert(sub(tag));
            return true;
          }
          if (tagIs(tag, "TD") && klasa(tag, "td-pager"))
          {
            znajdz_nastepne(sub(tag));
            return true;
          }
          return false;
        }
      });
      if (nast!="" && nast!=null)
;//!!!        Sciagacz.sciagaj(absoluteUrl(nast), new Rodzaj(C_LISTA_AUKCJI));
    }

//////////////////////////////////////////////////
//						//
//		LISTA KATEGORII			//
//						//
//////////////////////////////////////////////////
    

    void parsuj_liste_kategorii(NodeIterator it) throws ParserException
    {
      znajdz(it, new Warunek()
      {
        public boolean czy(Tag tag) throws ParserException
        {
          if (tagIs(tag, "TD") && klasa(tag, "small"))
              return true;
          String href = tag.getAttribute("href");
          if (tagIs(tag, "A") && href!=null)
          {
            if (href.indexOf("12")>=0) ///!!!!!!!!!
;//!!!              Sciagacz.sciagaj(absoluteUrl(href), new Rodzaj(C_LISTA_AUKCJI));
            return true;
          }
          return false;
        }
      });
    }

//////////////////////////////////////////////////
//						//
//		AUKCJA				//
//						//
//////////////////////////////////////////////////

  String rozpoznaj_cene(String napis)
  {
    int s1 = do_cyfry(napis, 0);
    int k1 = do_konca(napis, s1);
    return popraw_cene(napis.substring(s1, k1));
  }

  String rozpoznaj_sprzedawce(NodeIterator it) throws ParserException
  {
    ret_str = "";
    znajdz(it, new Warunek()
    {
        public boolean czy(Tag tag) throws ParserException
        {
          if (tagIs(tag, "SPAN") && klasa(tag, "uname"))
          {
              ret_kom.setUidWystawcy( url2uid(rozpoznaj_linka(sub(tag))) );
              ret_str = rozpoznaj_tresc(sub(tag));
              return true;
          }
          return false;
        }
    });
    return ret_str;
  }

  void parsuj_wiersz_oferty(NodeIterator it) throws ParserException
  {
    int ile = 0;
    String rodzaj = "";
    while (it.hasMoreNodes())
    {
      Node node = it.nextNode();
      if (!isTag(node)) continue;
      Tag tag = (Tag)node;
      if (!tagIs(tag, "TD")) continue;
      ++ile;
      String napis = rozpoznaj_tresc(sub(tag));
      if (ile==1)
        rodzaj = napis;
      if (ile!=2)
        continue;
      if (rodzaj.indexOf("Aktualna cena")>=0)
      {
        ret_of.put("cena", rozpoznaj_cene(napis));
        continue;
      }
      if (rodzaj.indexOf("Kup Teraz")>=0)
      {
        ret_of.put("cena_kup_teraz", rozpoznaj_cene(napis));
        continue;
      }
      if (rodzaj.indexOf("Do ko")>=0)
      {
        int x = napis.indexOf('(');
        int y = napis.indexOf(')');
        if (x<0 || y<0) continue;
        ret_of.put("do_konca", napis.substring(0, x).trim());
        ret_of.put("czas_konca", popraw_czas(napis.substring(x+1, y)));
        continue;
      }
      if (rodzaj.indexOf("Sprzedaj")>=0)
      {
        ret_of.put("sprzedawca", rozpoznaj_sprzedawce(sub(tag)));
        continue;
      }
      if (rodzaj.indexOf("Lokalizacja")>=0)
      {
        ret_of.put("lokalizacja", napis);
        continue;
      }
      if (rodzaj.indexOf("Transport")>=0)
      {
        ret_of.put("transport", napis);
        continue;
      }
      if (rodzaj.indexOf("Akceptowane formy p")>=0)
      {
        ret_of.put("platnosc", napis);
        continue;
      }
      if (rodzaj.indexOf("pna liczba")>=0)
      {
        ret_of.put("liczba_przedmiotow", napis);
        continue;
      }
      if (rodzaj.indexOf("Ofert kupna")>=0)
      {
        ret_of.put("ile_ofert", popraw_kreske(napis));
        continue;
      }
    }
  }

  void parsuj_wstep_oferty(NodeIterator it) throws ParserException
  {
    znajdz_wewnetrzne(it, new Warunek()
    {
      public boolean czy(Tag tag) throws ParserException
      {
        return tagIs(tag, "TR");
      }
      public void ok(Tag tag) throws ParserException
      {
        parsuj_wiersz_oferty(sub(tag));
      }
    });
  }


  void parsuj_atrybut(NodeIterator it) throws ParserException
  {
    int ile = 0;
    String rodzaj = "";
    while (it.hasMoreNodes())
    {
      Node node = it.nextNode();
      if (!isTag(node)) continue;
      Tag tag = (Tag)node;
      if (!tagIs(tag, "TD")) continue;
      ++ile;
      String napis = rozpoznaj_tresc(sub(tag));
      if (ile==1)
        rodzaj = napis;
      if (ile==2)
        ret_of.put(rodzaj, napis);
    }
  }


  void parsuj_atrybuty_oferty(NodeIterator it) throws ParserException
  {
    znajdz(it, new Warunek()
    {
        public boolean czy(Tag tag) throws ParserException
        {
          if (tagIs(tag, "TR") && klasa(tag, "attribs"))
          {
              parsuj_atrybut(sub(tag));
              return true;
          }
          return false;
        }
    });
  }

  String nazwa_kat;

  void parsuj_nazwe_kategorii(NodeIterator it) throws ParserException
  {
    nazwa_kat = "";
    znajdz(it, new Warunek()
    {
        public boolean czy(Tag tag) throws ParserException
        {
          if (tagIs(tag, "A") && klasa(tag, "bold10"))
          {
              if (!nazwa_kat.equals(""))
                nazwa_kat += " > ";
              nazwa_kat += rozpoznaj_tresc(sub(tag));
              return true;
          }
          return false;
        }
    });
    ret_of.put("kategoria", nazwa_kat);
  }

  String opis;

  void parsuj_oferte_dzialaj(NodeIterator it, String url) throws ParserException
  {
    opis = "";
    ret_of = new HashMap();
    ret_of.put("url", url);
    znajdz(it, new Warunek()
    {
        int stan = 0;
        int reszta = 0;
        public boolean czy(Tag tag) throws ParserException
        {
          if (tagIs(tag, "TD") && klasa(tag, "navbar-text")) // zawiera nazwe kategorii
          {
              parsuj_nazwe_kategorii(sub(tag));
              return true;
          }
          if (tagIs(tag, "TABLE") && klasa(tag, "it-main")) // zawiera cene
          {
              parsuj_wstep_oferty(sub(tag));
              return true;
          }
          if (tagIs(tag, "TD") && klasa(tag, "attribs1")) //atrybuty (opis - parametry)
          {
              parsuj_atrybuty_oferty(sub(tag));
              return true;
          }
          if (reszta==1 && tagIs(tag, "TR"))
          {		// fragmenty rozwalonej tabelki pod cen
            parsuj_wiersz_oferty(sub(tag));
            return true;
          }
          return false;
        }

        public void text(Text text) throws ParserException
        {
          if (stan==1)
          {
            opis += text.getText();
          }
          if (stan==2)
          {
            if (text.getText().equals("Tre opisu"))
              stan = 1;
          }
        }

        public void remark(Remark tag) throws ParserException
        {
          if (stan==1 && tag.getText().indexOf("end description (1)")>=0) stan = 3;
        }
        
        public void after(Tag tag) throws ParserException
        {
          if (stan==0 && tagIs(tag, "TABLE") && klasa(tag, "item-page"))
          {    // pasek z napisem "OPIS"
              stan = 1;
              //System.out.println("Wlaczam 1");
          }
          if (tagIs(tag, "TD") && klasa(tag, "attribs1"))  // atrybuty ( opis - parametry), omi
              stan = 2;
          if (tagIs(tag, "TABLE") && reszta==1)	
              // koniec rozwalonej tabelki z reszta rzeczy pod cen
              reszta = 0;
          if (tagIs(tag, "TABLE") && klasa(tag, "it-main")) // zawiera cene, pod spodem sa inne rzeczy
              reszta = 1;
        }
    });
    ret_of.put("opis", opis);
  }

  void parsuj_oferte(NodeIterator it, String url) throws ParserException
  {
    parsuj_oferte_dzialaj(it, url);
    fajny_parser.siets.doUpdate(url, ret_of);
    wyswietl();
  }


//////////////////////////////////////////////////
//						//
//		USER				//
//						//
//////////////////////////////////////////////////
  
  Komentarz ret_kom;

  void uzupelnij_cene_i_kategorie(Map map)
  {
    Object obj = map.get("cena");
    if (obj!=null)
      ret_kom.setCena( cena2liczba((String)obj) );
    if (ret_kom.getCena()<0)
    {
      obj = map.get("cena_kup_teraz");
      if (obj!=null)
        ret_kom.setCena( cena2liczba((String)obj) );
    }
    obj = map.get("kategoria");
    if (obj!=null)
      ret_kom.setKategoria( baza.kategoria2numer((String)obj) );
    if (ret_kom.getCena()<0 || ret_kom.getKategoria()<0)
      ++ile_porazek;
  }

  int cena2liczba(String s)
  {
    int x = s.indexOf(".");
    String s2 = s;
    if (x>=0)
      s2 = s.substring(0, x)+s.substring(x+1, s.length());
    try
    {
      return Integer.parseInt(s2);
    }
    catch (NumberFormatException e)
    {
      return -1;
    }
  }

  int url2nr_aukcji(String s)
  {
    int x = s.indexOf("item=");
    if (x<0) return -1;
    return rozpoznaj_liczbe(s, x, -1);
  }

  public int url2uid(String s)
  {
    int x = s.indexOf("uid=");
    if (x<0) return -1;
    return rozpoznaj_liczbe(s, x, -1);
  }

  int rozpoznaj_liczbe(String s, int pos0, int def)
  {
    int s1 = do_cyfry(s, pos0);
    int k1 = do_konca(s, s1);
    try
    {
      return Integer.parseInt(s.substring(s1, k1).trim());
    }
    catch (NumberFormatException e)
    {
      return def;
    }
  }

  boolean czy_sprz_ok;
  void rozpoznaj_uid_i_typ_usera(NodeIterator it) throws ParserException
  {
    czy_sprz_ok = false;
    znajdz(it, new Warunek()
    {
      public boolean czy(Tag tag) throws ParserException
      {
        if (tagIs(tag, "SPAN") && klasa(tag, "uname"))
        {
          ret_kom.setUidWystawcy( url2uid(rozpoznaj_linka(sub(tag))) );
          return true;
        }
        return false;
      }
      public void text(Text text) throws ParserException
      {
        boolean sprz = text.getText().indexOf(fajny_parser.mapa.get("kom_kupujacy"))>=0;
        boolean kup = text.getText().indexOf(fajny_parser.mapa.get("kom_sprzedajacy"))>=0;
        if (sprz || kup)
        {
          ret_kom.setCzySprzedawal(sprz);
          czy_sprz_ok = true;
        }
      }
    });
    if (!czy_sprz_ok)
       System.out.println("BLAD! Nie wiem czy kupujacy czy sprzedajacy");
  }

  int ile_porazek;

  void dodajKomentarz(String url_aukcji)
  {
    if (baza.czyWBazie(ret_kom))
      return;
    Map map = fajny_parser.siets.doRetrieve(url_aukcji);
    if (map!=null)
      uzupelnij_cene_i_kategorie(map);
    else 
    if (ile_porazek<U_max_por)
    {
      if (fajny_parser.sciagaj_aukcje_dla_kom(url_aukcji))
        uzupelnij_cene_i_kategorie(fajny_parser.ret_of);
      else
        ile_porazek++;
    }
    baza.wstawKomentarz(ret_kom);
  }

  int uid_celu;
  int ile_kom;
  boolean czy_ok;
  
  public FajnyParser fajny_parser;

  public String parsuj_usera(NodeIterator it, String url) throws ParserException
  {
    ile_porazek = 0;
    uid_celu = url2uid(url);
    ile_kom = 0;
    czy_ok = false;
    nast = "";
    if (uid_celu<0) return "";
    znajdz(it, new Warunek()
    {
        int item_type = 0;
        int ile = 0;
        int to_liczba_kom = 0;
        int ktore = 0;
        public boolean czy(Tag tag) throws ParserException
        {
          int t0 = -1;
          if (klasa(tag, "pos")) t0 = Api.KOM_POS;
          if (klasa(tag, "neu")) t0 = Api.KOM_NEU;
          if (klasa(tag, "neg")) t0 = Api.KOM_NEG;
          if (tagIs(tag, "SPAN") && t0>=0)
          {
            czy_ok = true;
            if (ktore<12 && ktore%4==2)
            {
              String s = rozpoznaj_tresc(sub(tag));
              ile_kom += rozpoznaj_liczbe(s, 0, 1000000);
            }
            if (ile==2)
              ret_kom.setPozytywny(t0);
            return true;
          }
          if (!tagIs(tag, "TD"))
            return false;
          if (klasa(tag, "td-pager"))
          {
            znajdz_nastepne(sub(tag));
            return true;
          }
          int typ = 0;
          if (klasa(tag, "list-color")) typ = 1;
          if (klasa(tag, "list-white")) typ = 2;
          if (typ==0) return false;
          if (item_type!=typ)
          {
            ile = 0;
            item_type = typ;
            ret_kom = new Komentarz();
	    ret_kom.setUidCelu(uid_celu);
          }
          ++ile;
          if (ile==1)
            rozpoznaj_uid_i_typ_usera(sub(tag));
          if (ile==2)
            return false;
          if (ile==3)
            ret_kom.setData( parsuj_czas(rozpoznaj_tresc(sub(tag))) );
          if (ile==4)
          {
            String aukcja = rozpoznaj_linka(sub(tag));
            ret_kom.setNrAukcji( url2nr_aukcji(aukcja) );
            dodajKomentarz(fajny_parser.absoluteUrl(aukcja));
          }
          return true;
        }
    });
    if (!czy_ok) throw new ParserException();
    int ile_w_bazie = baza.ileKomentarzyOUserze(uid_celu);
    System.out.println("ile_kom = "+ile_kom);
    ile_porazek = 0;
    return (ile_w_bazie==ile_kom) ? "" : nast;
  }

//////////////////////////////////////////////////
//						//
//		OGLNE				//
//						//
//////////////////////////////////////////////////

  public MadryParser(FajnyParser p)
  {
    fajny_parser = p;
  }

  public void wyswietl_strone(NodeIterator it) throws ParserException
  {
    znajdz(it, new Warunek()
    {
        public boolean czy(Tag tag) throws ParserException
        {
          String klasa = tag.getAttribute("class");
	  System.out.println("<"+tag.getTagName()+
	    (klasa==null ? "" : " class=\""+klasa+"\"") +">");
          return false;
        }

        public void remark(Remark t) throws ParserException
        {
          System.out.println("<!-- (remark) "+t.getText()+" -->");
	}

        public void text(Text text) throws ParserException
        {
          System.out.println(text.getText());
	}

        public void after(Tag tag) throws ParserException
        {
	  System.out.println("</"+tag.getTagName()+">");
        }
    });
  }


}
